// Etoile.cpp: implementation of the Etoile class.
//
//////////////////////////////////////////////////////////////////////

#include "Etoile.h"

// Globals for optimisation
static char tps[]="00";
static char tps4[]="0000";
static char tps6[]="000000";

static unsigned int HRpasse=0;
static float coef;
static float cmag;
static float rmag;

	int RAh;
	int RAm;
	float RAs;
	bool DEsign;
	int Deg;
	int Arcmin;
	int Arcsec;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Etoile::Etoile()
{
}

Etoile::~Etoile()
{	delete matTransfo;
}

int Etoile::Lire(ifstream & catalogue)
// Lis les infos de l'toile dans le fichier et calcule x,y et z;
{

  #if defined( LINUX )

    char ligne[135];
	catalogue.read(ligne,132);
	//catalogue.get();
    //printf(ligne,"\n");
	strncpy(tps4,ligne,4);		// numero HR
	HR = atoi(tps4);
//	cout << tps4  << " ";

	if (HR>HRpasse)
	{	HRpasse+=1000;
		char p=37;
		printf("%i%c ",HRpasse/100,p);
	}	
      	strncpy(tps6,ligne+102,6);	// Mag
	Mag=atof(tps6);
//	cout << Mag << endl;	

	strncpy(Name,ligne+4,10);	// Nom
	Name[10]=0;
//	cout << Name << " ";

	strncpy(tps,ligne+75,2);	// RAh
	RAh=atoi(tps);
//	cout << RAh << " ";
	
	strncpy(tps,ligne+77,2);	// RAm
	RAm=atoi(tps);
//	cout << RAm << " ";

	strncpy(tps4,ligne+79,4);	// RAs
	RAs=atof(tps4);
//	cout << RAs << " ";

	DEsign = (ligne[83]=='-');	// signe de DE : true si "-"
//	cout << ligne[83] << " ";

	strncpy(tps,ligne+84,2);	// Deg
	Deg=atoi(tps);
//	cout << Deg << " ";

	strncpy(tps,ligne+86,2);	// Arcmin
	Arcmin=atoi(tps);
//	cout << Arcmin << " ";

	strncpy(tps,ligne+88,2);	// Arcsec
	Arcsec=atoi(tps);
//	cout << Arcsec << endl;

	SpType=ligne[129];			// SpType
//	cout << SpType << endl;

	#endif

#if defined( WIN32 )
    char ligne[132];
	catalogue.read(ligne,131);
	//catalogue.get();
    //printf(ligne,"\n");
	strncpy(tps4,ligne,4);		// numero HR
	HR = atoi(tps4);
//	cout << tps4  << " ";

	if (HR>HRpasse)
	{	HRpasse+=1000;
		char p=37;
		printf("%i%c ",HRpasse/100,p);
	}	
      	strncpy(tps6,ligne+102,6);	// Mag
	Mag=atof(tps6);
//	cout << Mag << endl;	

	strncpy(Name,ligne+4,10);	// Nom
	Name[10]=0;
//	cout << Name << " ";

	strncpy(tps,ligne+75,2);	// RAh
	RAh=atoi(tps);
//	cout << RAh << " ";
	
	strncpy(tps,ligne+77,2);	// RAm
	RAm=atoi(tps);
//	cout << RAm << " ";

	strncpy(tps4,ligne+79,4);	// RAs
	RAs=atof(tps4);
//	cout << RAs << " ";

	DEsign = (ligne[83]=='-');	// signe de DE : true si "-"
//	cout << ligne[83] << " ";

	strncpy(tps,ligne+84,2);	// Deg
	Deg=atoi(tps);
//	cout << Deg << " ";

	strncpy(tps,ligne+86,2);	// Arcmin
	Arcmin=atoi(tps);
//	cout << Arcmin << " ";

	strncpy(tps,ligne+88,2);	// Arcsec
	Arcsec=atoi(tps);
//	cout << Arcsec << endl;

	SpType=ligne[129];			// SpType
//	cout << SpType << endl;
#endif

	// Calc the RA and DE from the datas
	RaRad=RA_en_Rad(RAh,RAm,RAs);
	DecRad=DE_en_Rad(DEsign,Deg,Arcmin,Arcsec);

	// Calc the Cartesian coord with RA and DE
	RADE_to_XYZ(RaRad,DecRad,XYZ);
	XYZ*=RAYON;

	matTransfo=new float[16];	// Used to staore the precalc transfos matrix used to draw the star

	glLoadIdentity();			// Init the current matrix

	// Precalcul de la matrice de Positionnement/Rotation pour le dessin
	glTranslatef(XYZ[0],XYZ[1],XYZ[2]);
	glRotatef(RaRad*180/PI,0,1,0);
	glRotatef(DecRad*180/PI,-1,0,0);
	
	glGetFloatv(GL_MODELVIEW_MATRIX , matTransfo);	// Store the matrix


	switch (SpType)			//Couleur en fonction du type spectral
	{	case 'O':	RGB[0]=0.8f; RGB[1]=0.8f; RGB[2]=1.0f; break;
		case 'B':	RGB[0]=0.9f; RGB[1]=0.9f; RGB[2]=1.0f; break;
		case 'A':	RGB[0]=1.0f; RGB[1]=1.0f; RGB[2]=1.0f; break;
		case 'F':	RGB[0]=1.0f; RGB[1]=1.0f; RGB[2]=0.8f; break;
		case 'G':	RGB[0]=1.0f; RGB[1]=1.0f; RGB[2]=0.7f; break;
		case 'K':	RGB[0]=1.0f; RGB[1]=0.9f; RGB[2]=0.8f; break;
		case 'M':
		case 'C':
		case 'S':	RGB[0]=1.0f; RGB[1]=0.6f; RGB[2]=0.6f; break;
	}

	return 0;
}




void Etoile::Afficher(float MagLimite)
{	if (Mag>MagLimite) return;
	// Check if in the field of view, if not return
	if (acos((XYZVision[0]*XYZ[0]+XYZVision[1]*XYZ[1]+XYZVision[2]*XYZ[2])/RAYON)>ChampPrecalc)/*((float)Ouv+4)*PI*((float)winW/winH)/360*/ return;

	// Random coef for star twinkling
	coef=(double)rand()/RAND_MAX;

	// Formula 'taken' from the starchart programm
/*`s' size: this argument provides an alternative way of specifying magnitude 
	symbol sizes, using a formula proposed by Andrew Young in Sky and Telescope, 
	March 1990, p. 311, D = D0 * 10^(k*(v0 - v)). He proposes a k of between 0.07 
	and 0.16, with 0.11 as probably the best. If `s' uses 0.1 magnitude quanta as 
	does `m'. The default values for D0, v0, and k are 0.5 (points), the limiting 
	magnitude of the first main map, and 0.11, respectively. These may be altered 
	with the optional arguments to `s', which set D0, v0 and k in that order, e.g.
	`s 0.7 6.6' sets D0 to 0.7 point, and v0 to magnitude 6.6 so that a magnitude 
	6.6 star will be printed as 0.7 point in diameter, and k will remain 0.11. `s 
	0.7 6.6 0.07' will set D0 and v0 as above, and also set k to 0.07. */
	
	// Calculation of the demi-size of the star texture
	rmag =0.25*pow(10,(0.12*(MagLimite - Mag)))*(pow(Ouv,0.7)/6);
	// Calculation of the luminosity
	cmag=(MagLimite-Mag)/(MagLimite)+0.3-(Ouv/400+0.25)*coef-(MagLimite-Mag/2.)*coefciel;//0.5*pow(10,(0.07*(MagLimite - Mag)));
	glColor3fv(RGB*cmag);

	{	glPushMatrix();
		glMultMatrixf(matTransfo);			// recuperation de la matrice calcul au chargement
		glBegin(GL_TRIANGLE_STRIP);
			glTexCoord2i(1,0);				//Bas Droite
			glVertex3f(rmag,-rmag,0.0f);
			glTexCoord2i(0,0);				//Bas Gauche
			glVertex3f(-rmag,-rmag,0.0f);
			glTexCoord2i(1,1);				//Haut Droit
			glVertex3f(rmag,rmag,0.0f);
			glTexCoord2i(0,1);				//Haut Gauche
			glVertex3f(-rmag,rmag,0.0f);
		glEnd ();
		glPopMatrix();	
	}
	
	// Now unused....
	/*	point pour les etoiles faibles 
	if (Mag>6)
	{	cmag/=((float)Ouv/25);
		if (cmag>1.0) cmag=0.6;
		cmag-=coefciel;
		glColor3f(cmag*r, cmag*g, cmag*b);
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_BLEND);
		glPointSize(0.5);
		glBegin(GL_POINTS);
			glVertex3f(X,Y,Z);
		glEnd();
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		return;
	}*/

}
