// *********************************************************************
//							===============
//							   Stellarium
//							===============
// *********************************************************************
/*
   Copyright (C) 2001 Fabien Chreau chereau@free.fr

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   See gpl.txt for more information regarding the GNU General Public License.
*/
// *********************************************************************

// I'm trying to comment the source but my english isn't perfect, so be tolerant 
// and please correct all the mistakes you find. ;)

#include <glut.h>
#include <time.h>

#include "astroLib/AstroOps.h"
#include "astroLib/DateOps.h"

#include "tga.h"
#include "LesEtoiles.h"
#include "Affichage.h"
#include "Variables.h"
#include "LesConstellations.h"
#include "LesPlanetes.h"

// *********************************************************************
LesEtoiles VouteCeleste;			// Class to manage all the stars from HR catalog
LesConstellations ConstellCeleste;	// Class to manage the constellation boundary and name
LesPlanetes SystemePlanete;			// Class to manage the planets


// *********************//temporaire/////////***************************
unsigned int HRtemp=0;
char HRstemp[4];
///////////////////////

// *********************************************************************
void init();

// *******  Calc the Zenith position for the location and Time  ********
void CalculeZenith()
{	RaZenith = -AstroOps::greenwichSiderealTime(JDay) + LeLieu.longitude();
	RaZenith = AstroOps::normalizeRadians(RaZenith);
	DeZenith = Astro::PI_OVER_TWO - LeLieu.latitude();
	//printf("RAzenith=%f DEzenith=%f\n",RaZenith*180/PI,DeZenith*180/PI);
}

// ***************  Print a beautiful console logo !! ******************
void AfficheIntro(void)
{	printf("\n");
	printf(" ## ### ### #   #   ### ###  #  # # # #\n");
	printf(" #   #  ##  #   #   ### ##   #  # # ###\n");
	printf("##   #  ### ### ### # # # #  #  ### # #\n\n");
	printf("%s \n\n",appName);
};

// *********************  Take the current time  ***********************
void RegleHeure(void)
{	time_t rawtime;
	tm * ptm;
	time ( &rawtime );
	ptm = gmtime ( &rawtime );
	int annee=ptm->tm_year+1900;
	int mois=ptm->tm_mon+1;
	int jour=ptm->tm_mday;
	int heure=ptm->tm_hour;
	int minute=ptm->tm_min;
	int seconde=ptm->tm_sec;
	JDay=DateOps::dmyToDay(jour,mois,annee);	// store it in the global variable Jday
	JDay+=((double)heure*0.0416666667+((double)minute+(double)seconde/60)*0.0006944444);
	CalculeZenith();							// Calc the zenith position
}

// *********************************************************************
// Increment/decrement smoothly the vision field and position
void MajVariables(void)
{	if (deltaOuv)						// if we are zooming in or out
	{	Ouv+=deltaOuv;					// change current zoom
		if (Ouv < 0.001) Ouv-=deltaOuv;	// stop on the max and min limits
		if (Ouv > 100) Ouv-=deltaOuv;
	}
	if (deltaRa) AzVision-=deltaRa;		// if we are mooving in the Azimthal angle (left/right)
	if (deltaDe)
	{	AltVision+=deltaDe;
		if (AltVision>=PI/2) AltVision-=deltaDe;
		if (AltVision<-PI/2) AltVision-=deltaDe;
	}
	// to prevent strange behaviour, change the speed of mooving (IN REAL TIME)
	// in function of the field of view : more it is zoomed, more the mooving speed is low (in angle)
	float depl=0.4/(Fps+1);
	if (deltaRa<0)
	{	deltaRa = -depl*Ouv/30;
	}
	else 
	{	if (deltaRa>0)
		{	deltaRa = depl*Ouv/30;
		}
	}
	if (deltaDe<0) 
	{	deltaDe = -depl*Ouv/30;
	}
	else 
	{	if (deltaDe>0)
		{	deltaDe = depl*Ouv/30;
		}
	}
	if (deltaOuv<0)
	{	deltaOuv = -depl*Ouv*5;
	}
	else 
	{	if (deltaOuv>0)
		{	deltaOuv = depl*Ouv*5;
		}
	}
}

// *********************************************************************
// Moove the Camera to the aim
void GoToAim(void)
{
}

// *********************************************************************
// Increment time every second for real time mode and every frame for accelered time mode
void Majtemps(void)
{	frame++;
	DeltaTemps=glutGet(GLUT_ELAPSED_TIME)-timefr;
	timefr+=DeltaTemps;
	if (Flaganim)								// real time animation
	{	JDay+=SECONDE*(float)DeltaTemps/1000.0;
	}
	if (timefr-timebase > 1000) 
	{	Fps=frame*1000.0/(timefr-timebase);		// Calc the PFS rate
		frame = 0;
		timebase+=1000;
		//JDay+=SECONDE;
	}

	if (Flaganimmin)							// accelered time animation
	{	JDay+=MINUTE;
		SystemePlanete.Calculer(JDay,LeLieu);
	}
}

// *****************  Display all the text informations  ******************
void AfficheTexte(void)
{	setOrthographicProjection(winW, winH);	// 2D coordinate
	glPushMatrix();
	glLoadIdentity();
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);

	if (FlagSelect)							// A star is selected : we print the information
	{	glColor3f(0.5f,0.8f,0.4f);
		sprintf(infoEtoile,"Nom :%s",EtoileSelectionnee.Nom);	
		renderBitmapString(3,13+13,(void *)font,infoEtoile);
		
		sprintf(infoEtoile,"HR  : %.4d",EtoileSelectionnee.HR);
		renderBitmapString(3,26+13,(void *)font,infoEtoile);
		
		sprintf(infoEtoile,"RA  : %.2dh %.2dm %.2fs",EtoileSelectionnee.RAh,EtoileSelectionnee.RAm,EtoileSelectionnee.RAs);	
		renderBitmapString(3,39+13,(void *)font,infoEtoile);

		sprintf(infoEtoile,"DE  : %.2fdeg",EtoileSelectionnee.DE*180/PI);	
		renderBitmapString(3,52+13,(void *)font,infoEtoile);

		sprintf(infoEtoile,"Mag : %.2f",EtoileSelectionnee.Mag);	
		renderBitmapString(3,65+13,(void *)font,infoEtoile);	
	}

	if (FlagHelp) AfficheHelp();			// Draw the help screen
	if (FlagInfo) AfficheInfo();			// Draw the Information screen

	glColor3f(0.0f,0.8f,0.9f);				// Text color

	int jour,mois;							// Print the date and time
	long annee;
	DateOps::dayToDmy((long)JDay,jour,mois,annee);
	sprintf(sDate,"%.2d/%.2d/%.4d",jour,mois,(int)annee);
	double reste=JDay-(int)DateOps::dmyToDay(jour,mois,annee);
	double heure=reste*24;
	double minute=(double)(heure-(int)heure)*60;
	int seconde=(int)(double)(minute-(int)minute)*60;
	sprintf(sHeure,"%.2d:%.2d:%.2d TU  fd=%.3f",(int)heure,(int)minute,seconde, Ouv);
	if (winW>450) renderBitmapString((winW-strlen(appName)*8)/2,13,(void *)font,appName);
	char speed[10];
	sprintf(speed,"FPS : %4.2f",Fps);
	renderBitmapString(winW-12*8,13,(void *)font,speed);
	renderBitmapString(3,13,(void *)font,sDate);
	renderBitmapString(3+12*10,13,(void *)font,sHeure);
	// display the properties of the item of the menu which have the mouse over if it is the case
	if (FlagTexteMenu) renderBitmapString(3,winH-40,(void *)font,stexteMenu );
	glPopMatrix();
	resetPerspectiveProjection();			// Restore the other coordinate
}


/*******************  My coordinate conventions :  *******************

Always count the RA angle from z axis to x axis 

		y
		|
		|   +A
		|DE.|
		| . |
		|___|____x
       / .  | .
     / RA . |.
   /...... .|
 z
						*/

// ****************  Change the coord system to altazimutal  *********
void Passe_en_altazimutal()
{	glLoadIdentity();
	RADE_to_XYZ(AzVision, AltVision, XYZVisionAltAz);
	// Calc the equatorial coordinate of the direction of vision wich was in Altazimuthal coordinate
	XYZVision=XYZVisionAltAz;
	AltAz_to_equ(XYZVision,RaZenith,DeZenith);
	// Change it... the vertical direction is always the vector 0,1,0 in Altazimuthal coordinate
	gluLookAt(0, 0, 0, 			// Observer position
		      XYZVisionAltAz[0],XYZVisionAltAz[1],XYZVisionAltAz[2],	// direction of vision
			  0,1,0);			// Vertical vector
}

// *****************  Change the coord system to equatorial  **********
void Passe_en_equatorial()
{	glLoadIdentity();
	Passe_en_altazimutal();
	// Make 2 the 2 rotation to transform the Altazimuthal system to Equatorial system
	glRotatef(90-DeZenith*180/PI,1,0,0);
	glRotatef(RaZenith*180/PI,0,1,0);
}


// ************************  Main display loop  ************************
void glutDisplay(void)
{	// Update the field of view
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(Ouv, (double) winW / winH, 1, 10000);
	glutPostRedisplay();
	glMatrixMode(GL_MODELVIEW);

	// Update the position of observation and time etc...
	MajVariables();
	Majtemps();
	if (FlagDeplacement) GoToAim();
	CalculeZenith();

	// Execute all the drawing function in the correct order from the more far to nearest objects

	// ---- Equatorial Coordinates
	Passe_en_equatorial();
	if (FlagVoieLactee) AfficheVoieLactee();			// Draw the milky way --> init the buffers
	//ConstellCeleste.Afficher("UMa");					// Draw one constellation only
	if (FlagConst) ConstellCeleste.Afficher();			// Draw all the constellations
	if (FlagVoute) VouteCeleste.Afficher(7);			// Draw the stars
	if (FlagPlanetes) SystemePlanete.Afficher();		// Draw the planets
	if (FlagLignes) AfficheMeridiens();					// Draw the meridian lines
	if (FlagLignes) AfficheParalleles();				// Draw the parallel lines
	if (FlagEquateurCeleste) AfficheEquateurCeleste();	// Draw the celestial equator line
	if (FlagEcliptique) AfficheEcliptique();			// Draw the ecliptic line
	if (FlagNomConst) ConstellCeleste.AfficherNom();	// Draw the names of the constellations
	if (FlagSelect) AffichePointeur(Pointeur*10,EtoileSelectionnee.RGB);	// Draw the star pointer

	// ---- AltAzimutal Coordinates
	Passe_en_altazimutal();
	if (FlagLignesAzimuth) AfficheMeridiensAzimuth();	// Draw the "Altazimuthal meridian" lines
	if (FlagLignesAzimuth) AfficheParallelesAzimuth();	// Draw the "Altazimuthal parallel" lines
	if (FlagCiel) AfficheCiel();						// Draw the blue sky
	if (FlagSol) AfficheDecor(2);						// Draw the mountains
	if (FlagSol) AfficheSol();							// Draw the ground
	if (FlagHorizon) AfficheHorizon();					// Draw the fog
	if (FlagCardinaux) AfficheCardinaux();				// Daw the cardinal points

	// ---- 2D Displays
	if (FlagTexte) AfficheTexte();						// Print all the text
	if (FlagMenu) AfficheMenu();						// Draw the menu bar
	
	glutSwapBuffers();
}

// ************************  On resize  *******************************

void glutResize(int w, int h)
{	if (!h)
		return;
	winW = w;
	winH = h;
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glViewport(0, 0, winW, winH);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(Ouv, (double) winW / winH, 1, 10000);	// Update the ratio
	glutPostRedisplay();
	glMatrixMode(GL_MODELVIEW);
}

// *******************  Handle the Flag keys  **************************
void processNormalKeys(unsigned char key, int x, int y) 
{	switch (key) 			// toggle the different functions
	{	case 27 : exit(0); break;
		case 'Z' :
		case 'z' : FlagHorizon=!FlagHorizon; break;
		case 'S' :
		case 's' : FlagSol=!FlagSol; break;
		case 'V' :
		case 'v' : FlagVoieLactee=!FlagVoieLactee; break;
		case 'E' :
		case 'e' : FlagVoute=!FlagVoute; break;
		case 'L' :
		case 'l' : FlagLignes=!FlagLignes; break;
		case 'C' :
		case 'c' : FlagCiel=!FlagCiel; break;
		case 'K' :
		case 'k' : FlagConst=!FlagConst; break;
		case 'T' :
		case 't' : FlagTexte=!FlagTexte; break;
		case 'A' :
		case 'a' : FlagCardinaux=!FlagCardinaux; break;
		case 'R' :
		case 'r' : FlagEcliptique=!FlagEcliptique; break;
		case 'F' :
		case 'f' : FlagEquateurCeleste=!FlagEquateurCeleste; break;
		case 'J' :
		case 'j' : FlagLignesAzimuth=!FlagLignesAzimuth; break;
		case '+' : coefciel+=0.01;
					if (coefciel>1) coefciel=1; break;
		case '-' : coefciel-=0.01;
					if (coefciel<0) coefciel=0; break;
		case '.' : Flaganim=!Flaganim; break;
		case '0' : Flaganimmin=!Flaganimmin; break;
		case 'N' :
		case 'n' : FlagNomConst=!FlagNomConst; break;
		case 'M' :
		case 'm' : FlagMenu=!FlagMenu; 
				   FlagTexteMenu = !FlagTexteMenu; break;
		case 'H' :
		case 'h' : FlagHelp=!FlagHelp;
					if (FlagInfo) FlagInfo=false;
					if (!FlagTexte) FlagTexte=true;
					break;
		case 'I' :
		case 'i' : FlagInfo=!FlagInfo; 
					if (FlagHelp) FlagHelp=false;
					break;
		case 'P' :
		case 'p' : FlagPlanetes=!FlagPlanetes; 
					break;
/*		case 'O' : //temporaire !! //////////////////
		case 'o' : 	ofstream ficconst ("consteltemp.fab",ios::app);
					if (ficconst.is_open())
					{	sprintf(HRstemp,"%.4d",HRtemp);
						ficconst.write(HRstemp,sizeof(HRstemp));
						ficconst.put(' ');
						ficconst.close();
					}  
					break;
					/////////////////////////////////////*/
	}
}

// ************************  Initialisation  ******************************
void glInit (void)
{	printf("Loading textures...\n");
	tga_t image;
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	//glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);		// init the blending function parameters (read the doc to understand it ;) ...)

	// If the pictures are in 32 bit format --> Alpha
	texIds[0] = image.GenId(); // let OpenGL give us a texture id
	if (image.Load("../textures/etoile16x16Aigrettes.tga", texIds[0]) == 0) // load image
		printf("Couldn't load texture id 0!!\n");
	texIds[1] = image.GenId(); // let OpenGL give us a texture id
	if (image.Load("../textures/sol2.tga", texIds[1]) == 0) // load image
		printf("Couldn't load texture id 1!!\n");
	texIds[2] = image.GenId(); // let OpenGL give us a texture id
	if (image.Load("../textures/voielactee256x256.tga", texIds[2]) == 0) // load image
		printf("Couldn't load texture id 2!!\n");
	texIds[3] = image.GenId(); // let OpenGL give us a texture id
	if (image.Load("../textures/horizon3.tga", texIds[3]) == 0) // load image
		printf("Couldn't load texture id 3!!\n");
	texIds[4] = image.GenId(); // let OpenGL give us a texture id
	if (image.Load("../textures/ciel.tga", texIds[4]) == 0) // load image
		printf("Couldn't load texture id 4!!\n");
		
	texIds[5] = image.GenId();
	
	texIds[6] = image.GenId();
	if (image.Load("../textures/N.tga", texIds[6]) == 0) printf("Couldn't load texture id 6!!\n");
	texIds[7] = image.GenId();
	if (image.Load("../textures/S.tga", texIds[7]) == 0) printf("Couldn't load texture id 7!!\n");
	texIds[8] = image.GenId();
	if (image.Load("../textures/E.tga", texIds[8]) == 0) printf("Couldn't load texture id 8!!\n");
	texIds[9] = image.GenId();
	if (image.Load("../textures/O.tga", texIds[9]) == 0) printf("Couldn't load texture id 9!!\n");
	texIds[10] = image.GenId();
	if (image.Load("../textures/Zenith.tga", texIds[10]) == 0) printf("Couldn't load texture id 10!!\n");
	texIds[11] = image.GenId();
	if (image.Load("../textures/Nadir.tga", texIds[11]) == 0) printf("Couldn't load texture id 11!!\n");
	texIds[12] = image.GenId();
	if (image.Load("../textures/pointeur3.tga", texIds[12]) == 0) printf("Couldn't load texture id 12!!\n");
	texIds[13] = image.GenId();
	if (image.Load("../textures/Bouton1a2.tga", texIds[13]) == 0) printf("Couldn't load texture id 13!!\n");
	texIds[14] = image.GenId();
	if (image.Load("../textures/Bouton2a2.tga", texIds[14]) == 0) printf("Couldn't load texture id 14!!\n");
	texIds[15] = image.GenId();
	if (image.Load("../textures/Bouton3a2.tga", texIds[15]) == 0) printf("Couldn't load texture id 15!!\n");
	texIds[16] = image.GenId();
	if (image.Load("../textures/Bouton4a2.tga", texIds[16]) == 0) printf("Couldn't load texture id 16!!\n");
	texIds[17] = image.GenId();
	if (image.Load("../textures/Bouton5a2.tga", texIds[17]) == 0) printf("Couldn't load texture id 17!!\n");
	texIds[18] = image.GenId();
	if (image.Load("../textures/Bouton6a2.tga", texIds[18]) == 0) printf("Couldn't load texture id 18!!\n");
	texIds[19] = image.GenId();
	if (image.Load("../textures/Bouton7a2.tga", texIds[19]) == 0) printf("Couldn't load texture id 19!!\n");
	texIds[20] = image.GenId();
	if (image.Load("../textures/Bouton8a2.tga", texIds[20]) == 0) printf("Couldn't load texture id 20!!\n");
	texIds[21] = image.GenId();
	if (image.Load("../textures/Bouton9a2.tga", texIds[21]) == 0) printf("Couldn't load texture id 21!!\n");
	texIds[22] = image.GenId();
	if (image.Load("../textures/Bouton10a2.tga", texIds[22]) == 0) printf("Couldn't load texture id 22!!\n");
	texIds[23] = image.GenId();
	if (image.Load("../textures/Bouton11a2.tga", texIds[23]) == 0) printf("Couldn't load texture id 23!!\n");

	texIds[25] = image.GenId();
	if (image.Load("../textures/etoile16x16.tga", texIds[25]) == 0) printf("Couldn't load texture id 22!!\n");


	texIds[40] = image.GenId();
	if (image.Load("../textures/sun.tga", texIds[40]) == 0) printf("Couldn't load texture id 40!!\n");
	texIds[41] = image.GenId();
	if (image.Load("../textures/mercure.tga", texIds[41]) == 0) printf("Couldn't load texture id 41!!\n");
	texIds[42] = image.GenId();
	if (image.Load("../textures/venus.tga", texIds[42]) == 0) printf("Couldn't load texture id 42!!\n");
	texIds[44] = image.GenId();
	if (image.Load("../textures/mars.tga", texIds[44]) == 0) printf("Couldn't load texture id 44!!\n");
	texIds[45] = image.GenId();
	if (image.Load("../textures/jupiter.tga", texIds[45]) == 0) printf("Couldn't load texture id 45!!\n");
	texIds[46] = image.GenId();
	if (image.Load("../textures/saturn.tga", texIds[46]) == 0) printf("Couldn't load texture id 46!!\n");
	texIds[47] = image.GenId();
	if (image.Load("../textures/saturneAnneaux128x128.tga", texIds[47]) == 0) printf("Couldn't load texture id 47!!\n");
	texIds[48] = image.GenId();
	if (image.Load("../textures/halo.tga", texIds[48]) == 0) printf("Couldn't load texture id 48!!\n");
	texIds[49] = image.GenId();
	if (image.Load("../textures/lune.tga", texIds[49]) == 0) printf("Couldn't load texture id 49!!\n");
	texIds[50] = image.GenId();
	if (image.Load("../textures/haloLune.tga", texIds[50]) == 0) printf("Couldn't load texture id 50!!\n");



	texIds[31] = image.GenId();
	if (image.Load("../textures/montagne1.tga", texIds[31]) == 0) printf("Couldn't load texture montagne1.tga!!\n");
	texIds[32] = image.GenId();
	if (image.Load("../textures/montagne2.tga", texIds[32]) == 0) printf("Couldn't load texture montagne2.tga!!\n");
	texIds[33] = image.GenId();
	if (image.Load("../textures/montagne3.tga", texIds[33]) == 0) printf("Couldn't load texture montagne3.tga!!\n");
	texIds[34] = image.GenId();
	if (image.Load("../textures/montagne4.tga", texIds[34]) == 0) printf("Couldn't load texture montagne4.tga!!\n");

}
// ********************  Handle the special keys  **********************
void pressKey(int key, int x, int y) 
{	float depl=0.4/(Fps+1);			// depl is used to maintain a constant speed whatever is the FPS
	switch (key) 
	{	// Direction and zoom deplacements
		case GLUT_KEY_LEFT : deltaRa = -depl*((float)Ouv/30);break;
		case GLUT_KEY_RIGHT : deltaRa = depl*((float)Ouv/30);break;
		case GLUT_KEY_UP : deltaDe = depl*((float)Ouv/30);break;
		case GLUT_KEY_DOWN : deltaDe = -depl*((float)Ouv/30);break;
		case GLUT_KEY_PAGE_UP : deltaOuv = -depl*Ouv/20;break;
		case GLUT_KEY_PAGE_DOWN : deltaOuv = depl*Ouv/20;break;
		// Switch to "windowed" Fullscreen mode (not a real fullscreen mode, only hide the desktop)
		case GLUT_KEY_F1:  
			glutFullScreen();
			break;
		// Switch to windowed mode
		case GLUT_KEY_F2:  
			winW = 640;winH = 480;
			glutReshapeWindow(winW,winH);
			break;
			
		// Switch to real fullscreen mode with different parameters (not very stable)
		case GLUT_KEY_F3:
			glutGameModeString("640x480:16");				// define resolution, color depth
			if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))	// enter full screen
			{	glutEnterGameMode();
				sprintf(gameModeString,"640x480:32");		// register callbacks again 
				init();										// and init OpenGL context
			}
			else glutGameModeString(gameModeString);
			break;
		case GLUT_KEY_F4:
			glutGameModeString("800x600:16");
			if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) 
			{	glutEnterGameMode();
				sprintf(gameModeString,"640x480:32");
				init();
			}
			else glutGameModeString(gameModeString);
			break;
		case GLUT_KEY_F5:
			glutGameModeString("1024x768:16");
			if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) 
			{	glutEnterGameMode();
				sprintf(gameModeString,"640x480:32");
				init();
			}
			else glutGameModeString(gameModeString);
			break;
		case GLUT_KEY_F6:
			glutGameModeString("640x480:32");
			if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) 
			{	glutEnterGameMode();
				sprintf(gameModeString,"640x480:32");
				init();
			}
			else glutGameModeString(gameModeString);
			break;
		case GLUT_KEY_F7:
			glutGameModeString("800x600:32");
			if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) 
			{	glutEnterGameMode();
				sprintf(gameModeString,"640x480:32");
				init();
			}
			else glutGameModeString(gameModeString);
			break;
		case GLUT_KEY_F8:
			glutGameModeString("1024x768:32");
			if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) 
			{	glutEnterGameMode();
				sprintf(gameModeString,"640x480:32");
				init();
			}
			else glutGameModeString(gameModeString);
			break;

		// Leave real fullscreen mode
		case GLUT_KEY_F9:  
			glutLeaveGameMode();
			break;
		}
}

// *******************  Stop mooving and zooming  **********************
void releaseKey(int key, int x, int y)
{	// When a deplacement key is released stop mooving
		switch (key)
	{	case GLUT_KEY_LEFT :
		case GLUT_KEY_RIGHT :	deltaRa = 0.0f;
								break;
		case GLUT_KEY_UP : 
		case GLUT_KEY_DOWN :	deltaDe = 0.0f;
								break;
		case GLUT_KEY_PAGE_UP :
		case GLUT_KEY_PAGE_DOWN :	deltaOuv = 0;
									break;
	}
}

// ******************  Change flags for the menu  **********************
void GereMenu(int x,int y)
{	int num=x/32+1;
	switch (num)
	{	case 1 : FlagConst=!FlagConst; return;
		case 2 : FlagNomConst=!FlagNomConst; return;
		case 3 : FlagLignes=!FlagLignes; return;
		case 4 : FlagLignesAzimuth=!FlagLignesAzimuth; return;
		case 5 : FlagCardinaux=!FlagCardinaux; return;
		case 6 : FlagSol=!FlagSol; return;
		case 7 : FlagHorizon=!FlagHorizon; return;
		case 8 : FlagCiel=!FlagCiel; return;
		case 9 : Flaganim=!Flaganim; return;
		case 10 : Flaganimmin=!Flaganimmin; return;
		case 11 : FlagTexte=!FlagTexte; return;
		case 12 : FlagHelp=!FlagHelp; return;
	}
}

// ***************  Handle the mouse deplacement  *********************
void BougeSouris(int x, int y)
{	if (y<winH-32 || x>384 || !FlagMenu)
	{	FlagTexteMenu=false;
		return;
	}
	FlagTexteMenu=true;		// If the menu is displayed
	int num=x/32+1;			// Calc the current picture under the mouse pointer
	switch (num)
	{	case 1 :  sprintf(stexteMenu,"Dessins des Constellations"); return;
		case 2 :  sprintf(stexteMenu,"Noms des Constellations"); return;
		case 3 :  sprintf(stexteMenu,"Grille Equatoriale"); return;
		case 4 :  sprintf(stexteMenu,"Grille Azimuthale"); return;
		case 5 :  sprintf(stexteMenu,"Points Cardinaux"); return;
		case 6 :  sprintf(stexteMenu,"Sol"); return;
		case 7 :  sprintf(stexteMenu,"Brume"); return;
		case 8 :  sprintf(stexteMenu,"Ciel (+ et - pour controler la luminositee)"); return;
		case 9 :  sprintf(stexteMenu,"Temps Reel"); return;
		case 10 : sprintf(stexteMenu,"Temps Accelere"); return;
		case 11 : sprintf(stexteMenu,"Texte"); return;
		case 12 : sprintf(stexteMenu,"Aide"); return;
	}
return;
}

// *******************  Handle the mouse clic  *************************
void Souris(int button, int state, int x, int y)
{	if (state) return;
	if (button==GLUT_RIGHT_BUTTON) 
	{	FlagSelect=false;
		sprintf(infoEtoile,"Selection : -");
		return;
	}
	else
	if (button==GLUT_LEFT_BUTTON)
	{	if (y>winH-32 && x<384)
		{	GereMenu(x,y);		// Menu
			return;
		}
		// Else try to select a star
		GLdouble M[16];	
		GLdouble P[16];
		GLdouble objx[1];
		GLdouble objy[1];
		GLdouble objz[1];
		GLint V[4];
		Passe_en_equatorial();
		// Convert x,y screen pos in 3D vector
		glGetDoublev(GL_MODELVIEW_MATRIX,M);
		glGetDoublev(GL_PROJECTION_MATRIX,P);
		glGetIntegerv(GL_VIEWPORT,V);
		if (gluUnProject(x,winH-y,1,M,P,V,objx,objy,objz))
		{	Pointeur[0]=*objx;
			Pointeur[1]=*objy;
			Pointeur[2]=*objz;
			if (!VouteCeleste.Rechercher(Pointeur))
			{	FlagSelect=true;
				VouteCeleste.InfoSelect(EtoileSelectionnee.XYZ,EtoileSelectionnee.RA,EtoileSelectionnee.DE,EtoileSelectionnee.Mag,EtoileSelectionnee.Nom,EtoileSelectionnee.HR,EtoileSelectionnee.RGB);
				Pointeur=EtoileSelectionnee.XYZ;
				RA_en_hms(EtoileSelectionnee.RAh,EtoileSelectionnee.RAm,EtoileSelectionnee.RAs,EtoileSelectionnee.RA);
			}
		}
	}
}

// ****************  Initialisation of glut and openGL  ****************
void init() 
{	//glutIgnoreKeyRepeat(1);
	glutKeyboardFunc(processNormalKeys);
	glutSpecialFunc(pressKey);
	glutSpecialUpFunc(releaseKey);
	glutDisplayFunc(glutDisplay);
	glutIdleFunc(glutDisplay);
	glutReshapeFunc(glutResize);
	glutMouseFunc(Souris);
	glutPassiveMotionFunc(BougeSouris);
//	glFrontFace(GL_CW);
//	glEnable(GL_CULL_FACE);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_BLEND);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
	glInit();
	glMatrixMode(GL_MODELVIEW);
	glutFullScreen();

}

// ***************************  Main  **********************************
void main (void)
{	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);
	glutInitWindowSize(winW, winH);
	glutCreateWindow(appName);
	AfficheIntro();
	RegleHeure();									// Pick current time
	VouteCeleste.Charger("../data/catalog.fab");	// Load stars
	ConstellCeleste.Charger("../data/constellationsmaj.fab",VouteCeleste);	// Load constellations
	SystemePlanete.Calculer(JDay,LeLieu);			// Compute planet data
	InitMeriParal();								// Precalculation for the grids
	init();

	glutMainLoop();									// Start drawing
}
